@@ -2,10 +2,10 @@
 
 from __future__ import annotations
 
-import argparse
 import logging as _logging
 import os
 import sys
+import warnings
 from multiprocessing import Event as mp_Event, Queue as mp_Queue, Value
 from operator import mul, setitem
 from os.path import abspath, basename, exists, isdir, isfile, islink, realpath
@@ -18,14 +18,31 @@ from urllib.parse import urlparse
 import PIL
 import requests
 
-from . import __version__, config, logging, notify, set_font_ratio, tui
+from . import (
+    FontRatio,
+    TermImageWarning,
+    config,
+    logging,
+    notify,
+    set_font_ratio,
+    tui,
+    utils,
+)
 from .config import config_options, store_config
-from .exceptions import URLNotFoundError
+from .exceptions import StyleError, TermImageError, URLNotFoundError
 from .exit_codes import FAILURE, INVALID_ARG, NO_VALID_SOURCE, SUCCESS
-from .image import _ALPHA_THRESHOLD, TermImage
+from .image import BlockImage, ITerm2Image, KittyImage, _best_style
 from .logging import Thread, init_log, log, log_exception
 from .logging_multi import Process
 from .tui.widgets import Image
+from .utils import (
+    CSI,
+    OS_IS_UNIX,
+    clear_queue,
+    get_terminal_size,
+    set_query_timeout,
+    write_tty,
+)
 
 
 def check_dir(
@@ -81,6 +98,8 @@ def check_dir(
     empty = True
     content = {}
     for entry in entries:
+        if interrupted and interrupted.is_set():
+            break
         if not SHOW_HIDDEN and entry.name.startswith("."):
             continue
         try:
@@ -336,7 +355,7 @@ def manage_checkers(
             setitem(checks_in_progress, *progress_queue.get())
 
             while not (
-                interrupted.is_set()  # MainThread has been interruped
+                interrupted.is_set()  # MainThread has been interrupted
                 or not any(checks_in_progress)  # All checkers are dead
                 # All checks are done
                 or (
@@ -378,6 +397,9 @@ def manage_checkers(
                 sleep(0.01)  # Allow queue sizes to be updated
         finally:
             if interrupted.is_set():
+                clear_queue(dir_queue)
+                clear_queue(content_queue)
+                clear_queue(progress_queue)
                 return
 
             if not any(checks_in_progress):
@@ -405,7 +427,7 @@ def manage_checkers(
         current_thread.name = "Checker"
 
         _, links, source, _depth = dir_queue.get()
-        while source:
+        while not interrupted.is_set() and source:
             log(f"Checking {source!r}", logger, verbose=True)
             if islink(source):
                 links.append((source, realpath(source)))
@@ -419,10 +441,13 @@ def manage_checkers(
                     source = abspath(source)
                     contents[source] = result
                     images.append((source, ...))
-                elif result is None:
+                elif not interrupted.is_set() and result is None:
                     log(f"{source!r} is empty", logger)
             _, links, source, _depth = dir_queue.get()
 
+        if interrupted.is_set():
+            clear_queue(dir_queue)
+
 
 def update_contents(
     dir: str,
@@ -463,13 +488,14 @@ def update_contents(
 def get_urls(
     url_queue: Queue,
     images: List[Tuple[str, Image]],
+    ImageClass: type,
 ) -> None:
     """Processes URL sources from a/some separate thread(s)"""
     source = url_queue.get()
     while not interrupted.is_set() and source:
         log(f"Getting image from {source!r}", logger, verbose=True)
         try:
-            images.append((basename(source), Image(TermImage.from_url(source))))
+            images.append((basename(source), Image(ImageClass.from_url(source))))
         # Also handles `ConnectionTimeout`
         except requests.exceptions.ConnectionError:
             log(f"Unable to get {source!r}", logger, _logging.ERROR)
@@ -483,16 +509,20 @@ def get_urls(
             log(f"Done getting {source!r}", logger, verbose=True)
         source = url_queue.get()
 
+    if interrupted.is_set():
+        clear_queue(url_queue)
+
 
 def open_files(
     file_queue: Queue,
     images: List[Tuple[str, Image]],
+    ImageClass: type,
 ) -> None:
     source = file_queue.get()
     while not interrupted.is_set() and source:
         log(f"Opening {source!r}", logger, verbose=True)
         try:
-            images.append((source, Image(TermImage.from_file(source))))
+            images.append((source, Image(ImageClass.from_file(source))))
         except PIL.UnidentifiedImageError as e:
             log(str(e), logger, _logging.ERROR)
         except OSError as e:
@@ -501,11 +531,18 @@ def open_files(
             log_exception(f"Opening {source!r} failed", logger, direct=True)
         source = file_queue.get()
 
+    if interrupted.is_set():
+        clear_queue(file_queue)
+
 
 def main() -> None:
     """CLI execution sub-entry-point"""
+    from .parsers import parser, style_parsers
+
     global args, url_images, MAX_DEPTH, RECURSIVE, SHOW_HIDDEN
 
+    warnings.filterwarnings("error", "", TermImageWarning, "term_image.image.iterm2")
+
     def check_arg(
         name: str,
         check: Callable[[Any], Any],
@@ -552,454 +589,6 @@ def main() -> None:
 
         return bool(valid)
 
-    parser = argparse.ArgumentParser(
-        prog="term-image",
-        formatter_class=argparse.RawDescriptionHelpFormatter,
-        description="Display/Browse images in a terminal",
-        epilog=""" \
-
-'--' should be used to separate positional arguments that begin with an '-' \
-from options/flags, to avoid ambiguity.
-For example, `$ term-image [options] -- -image.jpg --image.png`
-
-NOTES:
-  1. The displayed image uses HEIGHT/2 lines, while the number of columns is dependent
-     on the WIDTH and the FONT RATIO.
-     The auto sizing is calculated such that the image always fits into the available
-     terminal size (i.e terminal size minus allowances) except when `-S` is
-     specified, which allows the image height to go beyond the terminal height.
-  2. The size is multiplied by the scale on each axis respectively before the image
-     is rendered. A scale value must be such that 0.0 < value <= 1.0.
-  3. In CLI mode, only image sources are used, directory sources are skipped.
-     Animated images are displayed only when animation is disabled (with `--no-anim`)
-     or when there's only one image source.
-  4. Any image having more pixels than the specified maximum will be:
-     - skipped, in CLI mode, if '--max-pixels-cli' is specified.
-     - replaced, in TUI mode, with a placeholder when displayed but can still be forced
-       to display or viewed externally.
-     Note that increasing this should not have any effect on general performance
-     (i.e navigation, etc) but the larger an image is, the more the time and memory
-     it'll take to render it. Thus, a large image might delay the rendering of other
-     images to be rendered immediately after it.
-  5. Frames will not be cached for any animation with more frames than this value.
-     Memory usage depends on the frame count per image, not this maximum count.
-  6. Any event with a level lower than the specified one is not reported.
-  7. Supports all image formats supported by `PIL.Image.open()`.
-     See https://pillow.readthedocs.io/en/latest/handbook/image-file-formats.html for
-     details.
-""",
-        add_help=False,  # '-h' is used for HEIGHT
-    )
-
-    # General
-    general = parser.add_argument_group("General Options")
-
-    general.add_argument(
-        "--help",
-        action="help",
-        help="Show this help message and exit",
-    )
-    general.add_argument(
-        "--version",
-        action="version",
-        version=__version__,
-        help="Show the program version and exit",
-    )
-    general.add_argument(
-        "--reset-config",
-        action="store_true",
-        help="Restore default config and exit (Overwrites the config file)",
-    )
-    general.add_argument(
-        "-F",
-        "--font-ratio",
-        type=float,
-        metavar="N",
-        default=config.font_ratio,
-        help=(
-            "Specify the width-to-height ratio of a character cell in your terminal "
-            f"for proper image scaling (default: {config.font_ratio})"
-        ),
-    )
-    mode_options = general.add_mutually_exclusive_group()
-    mode_options.add_argument(
-        "--cli",
-        action="store_true",
-        help=(
-            "Do not the launch the TUI, instead draw all image sources "
-            "to the terminal directly [3]"
-        ),
-    )
-    mode_options.add_argument(
-        "--tui",
-        action="store_true",
-        help="Always launch the TUI, even for a single image",
-    )
-
-    # # Animation
-    anim_options = parser.add_argument_group("Animation Options (General)")
-    anim_options.add_argument(
-        "-f",
-        "--frame-duration",
-        type=float,
-        metavar="N",
-        help=(
-            "Specify the time (in seconds) between frames for all animated images "
-            "(default: Determined per image from it's metadata OR 0.1)"
-        ),
-    )
-    anim_options.add_argument(
-        "-R",
-        "--repeat",
-        type=int,
-        default=-1,
-        metavar="N",
-        help=(
-            "Number of times to repeat all frames of an animated image; A negative "
-            "count implies an infinite loop (default: -1)"
-        ),
-    )
-
-    anim_cache_options = anim_options.add_mutually_exclusive_group()
-    anim_cache_options.add_argument(
-        "--anim-cache",
-        type=int,
-        default=config.anim_cache,
-        metavar="N",
-        help=(
-            "Maximum frame count for animation frames to be cached (Better performance "
-            f"at the cost of memory) (default: {config.anim_cache}) [5]"
-        ),
-    )
-    anim_cache_options.add_argument(
-        "--cache-all-anim",
-        action="store_true",
-        help=(
-            "Cache frames for all animations (Beware, uses up a lot of memory for "
-            "animated images with very high frame count)"
-        ),
-    )
-    anim_cache_options.add_argument(
-        "--cache-no-anim",
-        action="store_true",
-        help="Disable frame caching (Less memory usage but reduces performance)",
-    )
-
-    anim_options.add_argument(
-        "--no-anim",
-        action="store_true",
-        help=(
-            "Disable image animation. Animated images are displayed as just their "
-            "first frame."
-        ),
-    )
-
-    # # Transparency
-    _alpha_options = parser.add_argument_group(
-        "Transparency Options (General)",
-        "NOTE: These are mutually exclusive",
-    )
-    alpha_options = _alpha_options.add_mutually_exclusive_group()
-    alpha_options.add_argument(
-        "--no-alpha",
-        action="store_true",
-        help="Disable image transparency (i.e black background)",
-    )
-    alpha_options.add_argument(
-        "-A",
-        "--alpha",
-        type=float,
-        metavar="N",
-        default=_ALPHA_THRESHOLD,
-        help=(
-            "Alpha ratio above which pixels are taken as opaque (0 <= x < 1) "
-            f"(default: {_ALPHA_THRESHOLD:f})"
-        ),
-    )
-    alpha_options.add_argument(
-        "-b",
-        "--alpha-bg",
-        metavar="COLOR",
-        help=(
-            "Hex color (without '#') with which transparent backgrounds should be "
-            "replaced"
-        ),
-    )
-
-    # CLI-only
-    cli_options = parser.add_argument_group(
-        "CLI-only Options",
-        "These options apply only when there is just one valid image source",
-    )
-
-    size_options = cli_options.add_mutually_exclusive_group()
-    size_options.add_argument(
-        "-w",
-        "--width",
-        type=int,
-        metavar="N",
-        help="Width of the image to be rendered (default: auto) [1]",
-    )
-    size_options.add_argument(
-        "-h",
-        "--height",
-        type=int,
-        metavar="N",
-        help="Height of the image to be rendered (default: auto) [1]",
-    )
-    cli_options.add_argument(
-        "--h-allow",
-        type=int,
-        default=0,
-        metavar="N",
-        help=(
-            "Horizontal allowance i.e minimum number of columns to leave unused "
-            "(default: 0)"
-        ),
-    )
-    cli_options.add_argument(
-        "--v-allow",
-        type=int,
-        default=2,
-        metavar="N",
-        help=(
-            "Vertical allowance i.e minimum number of lines to leave unused "
-            "(default: 2)"
-        ),
-    )
-    cli_options.add_argument(
-        "-S",
-        "--scroll",
-        action="store_true",
-        help=(
-            "Allow the image height to go beyond the terminal height. "
-            "Not needed when `--fit-to-width` is specified."
-        ),
-    )
-    size_options.add_argument(
-        "--fit-to-width",
-        action="store_true",
-        help=(
-            "Automatically fit the image to the available terminal width. "
-            "`--v-allow` has no effect i.e vertical allowance is overriden."
-        ),
-    )
-    cli_options.add_argument(
-        "-O",
-        "--oversize",
-        action="store_true",
-        help=(
-            "Allow the image size to go beyond the terminal size "
-            "(To be used with `-w` or `-h`)"
-        ),
-    )
-    cli_options.add_argument(
-        "-s",
-        "--scale",
-        type=float,
-        metavar="N",
-        help="Scale of the image to be rendered (overrides `-x` and `-y`) [2]",
-    )
-    cli_options.add_argument(
-        "-x",
-        "--scale-x",
-        type=float,
-        metavar="N",
-        default=1.0,
-        help="x-axis scale of the image to be rendered (default: 1.0) [2]",
-    )
-    cli_options.add_argument(
-        "-y",
-        "--scale-y",
-        type=float,
-        metavar="N",
-        default=1.0,
-        help="y-axis scale of the image to be rendered (default: 1.0) [2]",
-    )
-    cli_options.add_argument(
-        "--max-pixels-cli",
-        action="store_true",
-        help=("Apply '--max-pixels' in CLI mode"),
-    )
-
-    # # Alignment
-    align_options = parser.add_argument_group("Alignment Options (CLI-only)")
-    align_options.add_argument(
-        "--no-align",
-        action="store_true",
-        help=(
-            "Output image without alignment or padding. "
-            "Overrides all other alignment options"
-        ),
-    )
-    align_options.add_argument(
-        "-H",
-        "--h-align",
-        choices=("left", "center", "right"),
-        help="Horizontal alignment (default: center)",
-    )
-    align_options.add_argument(
-        "--pad-width",
-        metavar="N",
-        type=int,
-        help=(
-            "No of columns within which to align the image "
-            "(default: terminal width, minus horizontal allowance)"
-        ),
-    )
-    align_options.add_argument(
-        "-V",
-        "--v-align",
-        choices=("top", "middle", "bottom"),
-        help="Vertical alignment (default: middle)",
-    )
-    align_options.add_argument(
-        "--pad-height",
-        metavar="N",
-        type=int,
-        help=(
-            "No of lines within which to align the image "
-            "(default: terminal height, minus vertical allowance)"
-        ),
-    )
-
-    # TUI-only
-    tui_options = parser.add_argument_group(
-        "TUI-only Options",
-        (
-            "These options apply only when there is at least one valid directory source"
-            "or multiple valid sources"
-        ),
-    )
-
-    tui_options.add_argument(
-        "-a",
-        "--all",
-        action="store_true",
-        help="Inlcude hidden file and directories",
-    )
-    tui_options.add_argument(
-        "-r",
-        "--recursive",
-        action="store_true",
-        help="Scan for local images recursively",
-    )
-    tui_options.add_argument(
-        "-d",
-        "--max-depth",
-        type=int,
-        metavar="N",
-        default=sys.getrecursionlimit() - 50,
-        help=f"Maximum recursion depth (default: {sys.getrecursionlimit() - 50})",
-    )
-
-    # Performance
-    perf_options = parser.add_argument_group("Performance Options (General)")
-    perf_options.add_argument(
-        "--max-pixels",
-        type=int,
-        metavar="N",
-        default=config.max_pixels,
-        help=(
-            "Maximum amount of pixels in images to be displayed "
-            f"(default: {config.max_pixels}) [4]"
-        ),
-    )
-
-    perf_options.add_argument(
-        "--checkers",
-        type=int,
-        metavar="N",
-        default=config.checkers,
-        help=(
-            "Maximum number of sub-processes for checking directory sources "
-            f"(default: {config.checkers})"
-        ),
-    )
-    perf_options.add_argument(
-        "--getters",
-        type=int,
-        metavar="N",
-        default=config.getters,
-        help=(
-            "Number of threads for downloading images from URL sources "
-            "(default: {config.getters})"
-        ),
-    )
-    perf_options.add_argument(
-        "--grid-renderers",
-        type=int,
-        metavar="N",
-        default=config.grid_renderers,
-        help=(
-            "Number of subprocesses for rendering grid cells "
-            "(default: {config.grid_renderers})"
-        ),
-    )
-    perf_options.add_argument(
-        "--no-multi",
-        action="store_true",
-        help="Disable multiprocessing",
-    )
-
-    # Logging
-    log_options_ = parser.add_argument_group(
-        "Logging Options",
-        "NOTE: These are mutually exclusive",
-    )
-    log_options = log_options_.add_mutually_exclusive_group()
-
-    log_options_.add_argument(
-        "-l",
-        "--log-file",
-        metavar="FILE",
-        default=config.log_file,
-        help=f"Specify a file to write logs to (default: {config.log_file})",
-    )
-    log_options.add_argument(
-        "--log-level",
-        metavar="LEVEL",
-        choices=("DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"),
-        default="WARNING",
-        help=(
-            "Set logging level to any of DEBUG, INFO, WARNING, ERROR, CRITICAL "
-            "(default: WARNING) [6]"
-        ),
-    )
-    log_options.add_argument(
-        "-q",
-        "--quiet",
-        action="store_true",
-        help="No notifications, except fatal and config errors",
-    )
-    log_options.add_argument(
-        "-v",
-        "--verbose",
-        action="store_true",
-        help="More detailed event reporting. Also sets logging level to INFO",
-    )
-    log_options.add_argument(
-        "--verbose-log",
-        action="store_true",
-        help="Like --verbose but only applies to the log file",
-    )
-    log_options.add_argument(
-        "--debug",
-        action="store_true",
-        help="Implies --log-level=DEBUG with verbosity",
-    )
-
-    # Positional
-    parser.add_argument(
-        "sources",
-        nargs="*",
-        metavar="source",
-        help=(
-            "Path(s) to local image(s) and/or directory(s) OR URLs. "
-            "If no source is given, the current working directory is used."
-        ),
-    )
-
     args = parser.parse_args()
     MAX_DEPTH = args.max_depth
     RECURSIVE = args.recursive
@@ -1009,6 +598,10 @@ NOTES:
         store_config(default=True)
         sys.exit(SUCCESS)
 
+    force_cli_mode = not sys.stdout.isatty() and not args.cli
+    if force_cli_mode:
+        args.cli = True
+
     init_log(
         (
             args.log_file
@@ -1055,7 +648,81 @@ NOTES:
             )
             setattr(args, var_name, getattr(config, var_name))
 
-    set_font_ratio(args.font_ratio)
+    set_query_timeout(args.query_timeout)
+    utils.SWAP_WIN_SIZE = args.swap_win_size
+
+    if args.auto_font_ratio:
+        args.font_ratio = None
+    try:
+        set_font_ratio(args.font_ratio or FontRatio.FULL_AUTO)
+    except TermImageError:
+        notify.notify(
+            "Auto font ratio is not supported in the active terminal or on this "
+            "platform, using 0.5. It can be set otherwise using `-F | --font-ratio`.",
+            level=notify.WARNING,
+        )
+        args.font_ratio = 0.5
+
+    ImageClass = {
+        "auto": None,
+        "kitty": KittyImage,
+        "iterm2": ITerm2Image,
+        "block": BlockImage,
+    }[args.style]
+    if not ImageClass:
+        ImageClass = _best_style()
+    args.style = ImageClass.__name__[:-5].lower()
+
+    if args.force_style or args.style == config.style != "auto":
+        ImageClass.is_supported()  # Some classes need to set some attributes
+        ImageClass._supported = True
+    else:
+        try:
+            ImageClass(None)
+        except StyleError:  # Instantiation isn't permitted
+            write_tty(f"{CSI}1K\r".encode())  # Erase emitted APCs
+            log(
+                f"The {args.style!r} render style is not supported in the current "
+                "terminal! To use it anyways, add '--force-style'.",
+                logger,
+                level=_logging.CRITICAL,
+            )
+            return FAILURE
+        except TypeError:  # Instantiation is permitted
+            if not ImageClass.is_supported():  # Also sets any required attributes
+                write_tty(f"{CSI}1K\r".encode())  # Erase emitted APCs
+                log(
+                    f"The {args.style!r} render style might not be fully supported in "
+                    "the current terminal... using it anyways.",
+                    logger,
+                    level=_logging.WARNING,
+                )
+
+    # Some APCs (e.g kitty's) used for render style support detection get emitted on
+    # some non-supporting terminal emulators
+    write_tty(f"{CSI}1K\r".encode())  # Erase emitted APCs
+
+    log(f"Using {args.style!r} render style", logger, verbose=True)
+    style_parser = style_parsers.get(args.style)
+    style_args = vars(style_parser.parse_known_args()[0]) if style_parser else {}
+
+    if args.style == "iterm2":
+        ITerm2Image.JPEG_QUALITY = style_args.pop("jpeg_quality")
+        ITerm2Image.NATIVE_ANIM_MAXSIZE = style_args.pop("native_maxsize")
+        ITerm2Image.READ_FROM_FILE = style_args.pop("read_from_file")
+
+    try:
+        style_args = ImageClass._check_style_args(style_args)
+    except ValueError as e:
+        notify.notify(str(e), level=notify.CRITICAL)
+        return INVALID_ARG
+
+    if force_cli_mode:
+        log(
+            "Output is not a terminal, forcing CLI mode!",
+            logger,
+            level=_logging.WARNING,
+        )
 
     log("Processing sources", logger, loading=True)
 
@@ -1070,36 +737,30 @@ NOTES:
     getters = [
         Thread(
             target=get_urls,
-            args=(url_queue, url_images),
+            args=(url_queue, url_images, ImageClass),
             name=f"Getter-{n}",
-            daemon=True,
         )
         for n in range(1, args.getters + 1)
     ]
-    for getter in getters:
-        getter.start()
+    getters_started = False
 
     file_queue = Queue()
     opener = Thread(
         target=open_files,
-        args=(file_queue, file_images),
+        args=(file_queue, file_images, ImageClass),
         name="Opener",
-        daemon=True,
     )
-    opener.start()
+    opener_started = False
 
-    os_is_unix = sys.platform not in {"win32", "cygwin"}
-
-    if os_is_unix and not args.cli:
+    if OS_IS_UNIX and not args.cli:
         dir_queue = mp_Queue() if logging.MULTI and args.checkers > 1 else Queue()
         dir_queue.sources_finished = False
         check_manager = Thread(
             target=manage_checkers,
             args=(dir_queue, contents, dir_images),
             name="CheckManager",
-            daemon=True,
         )
-        check_manager.start()
+    checkers_started = False
 
     for source in sources:
         if source in unique_sources:
@@ -1108,41 +769,65 @@ NOTES:
         unique_sources.add(source)
 
         if all(urlparse(source)[:3]):  # Is valid URL
+            if not getters_started:
+                for getter in getters:
+                    getter.start()
+                getters_started = True
             url_queue.put(source)
         elif isfile(source):
+            if not opener_started:
+                opener.start()
+                opener_started = True
             file_queue.put(source)
         elif isdir(source):
             if args.cli:
                 log(f"Skipping directory {source!r}", logger, verbose=True)
                 continue
-            if not os_is_unix:
+            if not OS_IS_UNIX:
                 dir_images = True
                 continue
+            if not checkers_started:
+                check_manager.start()
+                checkers_started = True
             dir_queue.put(("", [], source, 0))
         else:
             log(f"{source!r} is invalid or does not exist", logger, _logging.ERROR)
 
     # Signal end of sources
-    for _ in range(args.getters):
-        url_queue.put(None)
-    file_queue.put(None)
-    if os_is_unix and not args.cli:
+    if getters_started:
+        for _ in range(args.getters):
+            url_queue.put(None)
+    if opener_started:
+        file_queue.put(None)
+    if checkers_started:
         if logging.MULTI and args.checkers > 1:
             dir_queue.sources_finished = True
         else:
             dir_queue.put((None,) * 4)
 
-    for getter in getters:
-        getter.join()
-    opener.join()
-    if os_is_unix and not args.cli:
-        check_manager.join()
+    interrupt = None
+    while True:
+        try:
+            if getters_started:
+                for getter in getters:
+                    getter.join()
+            if opener_started:
+                opener.join()
+            if checkers_started:
+                check_manager.join()
+            break
+        except KeyboardInterrupt as e:  # Ensure logs are in correct order
+            if not interrupt:  # keep the first
+                interrupted.set()
+                interrupt = e
+    if interrupt:
+        raise interrupt from None
 
     notify.stop_loading()
     while notify.is_loading():
         pass
 
-    if not os_is_unix and dir_images:
+    if not OS_IS_UNIX and dir_images:
         log(
             "Directory sources skipped, not supported on Windows!",
             logger,
@@ -1162,9 +847,12 @@ NOTES:
     ):
         log("Running in CLI mode", logger, direct=False)
 
+        if style_args.get("native") and len(images) > 1:
+            style_args["stall_native"] = False
+
         show_name = len(args.sources) > 1
         for entry in images:
-            image = entry[1]._image
+            image = entry[1]._ti_image
             if args.max_pixels_cli and mul(*image._original_size) > args.max_pixels:
                 log(
                     f"Has more than the maximum pixel-count, skipping: {entry[0]!r}",
@@ -1174,7 +862,12 @@ NOTES:
                 )
                 continue
 
-            if not args.no_anim and image._is_animated and len(images) > 1:
+            if (
+                not args.no_anim
+                and image._is_animated
+                and not style_args.get("native")
+                and len(images) > 1
+            ):
                 log(f"Skipping animated image: {entry[0]!r}", logger, verbose=True)
                 continue
 
@@ -1194,6 +887,27 @@ NOTES:
                 if args.frame_duration:
                     image.frame_duration = args.frame_duration
 
+                if args.style == "kitty":
+                    image.set_render_method(
+                        "lines"
+                        if (
+                            ImageClass._KITTY_VERSION
+                            and image._is_animated
+                            and not args.no_anim
+                        )
+                        else "whole"
+                    )
+                elif args.style == "iterm2":
+                    image.set_render_method(
+                        "whole"
+                        if (
+                            ImageClass._TERM == "konsole"
+                            # Always applies to non-native animations also
+                            or image.rendered_height <= get_terminal_size()[1]
+                        )
+                        else "lines"
+                    )
+
                 image.draw(
                     *(
                         (None, 1, None, 1)
@@ -1208,7 +922,9 @@ NOTES:
                     (
                         None
                         if args.no_alpha
-                        else args.alpha_bg and "#" + args.alpha_bg or args.alpha
+                        else (
+                            args.alpha if args.alpha_bg is None else "#" + args.alpha_bg
+                        )
                     ),
                     scroll=args.scroll,
                     animate=not args.no_anim,
@@ -1218,16 +934,17 @@ NOTES:
                         and (args.cache_all_anim or args.anim_cache)
                     ),
                     check_size=not args.oversize,
+                    **style_args,
                 )
 
-            # Handles `ValueError` and `.exceptions.InvalidSize`
-            # raised by `TermImage.set_size()`, scaling value checks
+            # Handles `ValueError` and `.exceptions.InvalidSizeError`
+            # raised by `BaseImage.set_size()`, scaling value checks
             # or padding width/height checks.
-            except ValueError as e:
+            except (ValueError, StyleError, TermImageWarning) as e:
                 notify.notify(str(e), level=notify.ERROR)
-    elif os_is_unix:
+    elif OS_IS_UNIX:
         notify.end_loading()
-        tui.init(args, images, contents)
+        tui.init(args, style_args, images, contents, ImageClass)
     else:
         log(
             "The TUI is not supported on Windows! Try with `--cli`.",
@@ -1245,18 +962,21 @@ logger = _logging.getLogger(__name__)
 # Will be updated from `.logging.init_log()` if multiprocessing is enabled
 interrupted: Union[None, Event, mp_Event] = None
 
+# The annotations below are put in comments for compatibility with Python 3.7
+# as it doesn't allow names declared as `global` within functions to be annotated.
+
 # Used by `check_dir()`
-_depth: int = None
+_depth = None  #: int
 
 # Set from within `check_dirs()`; Hence, only set in "Checker-?" processes
-_dir_queue: Union[None, Queue, mp_Queue] = None
-_free_checkers: Optional[Value] = None
-_source: Optional[str] = None
+_dir_queue = None  #: Union[None, Queue, mp_Queue]
+_free_checkers = None  #: Optional[Value]
+_source = None  #: Optional[str]
 
 # Set from within `main()`
-MAX_DEPTH: Optional[int] = None
-RECURSIVE: Optional[bool] = None
-SHOW_HIDDEN: Optional[bool] = None
+MAX_DEPTH = None  #: Optional[int]
+RECURSIVE = None  #: Optional[bool]
+SHOW_HIDDEN = None  #: Optional[bool]
 # # Used in other modules
-args: Optional[argparse.Namespace] = None
-url_images: Optional[list] = None
+args = None  #: Optional[argparse.Namespace]
+url_images = None  #: Optional[list]
