@@ -12,6 +12,7 @@ from typing import Any, Dict
 import urwid
 
 from .exit_codes import CONFIG_ERROR
+from .utils import COLOR_RESET, CSI, QUERY_TIMEOUT
 
 
 def action_with_key(key: str, keyset: Dict[str, list]) -> str:
@@ -24,18 +25,18 @@ def action_with_key(key: str, keyset: Dict[str, list]) -> str:
 
 def info(msg: str) -> None:
     print(
-        f"\033[34mconfig: \033[0m{msg}",
+        f"{CSI}34mconfig: {COLOR_RESET}{msg}",
         # In case output is being redirected or piped
         file=sys.stdout if sys.stdout.isatty() else sys.stderr,
     )
 
 
 def error(msg: str) -> None:
-    print(f"\033[34mconfig: \033[33m{msg}\033[0m", file=sys.stderr)
+    print(f"{CSI}34mconfig: {CSI}31m{msg}{COLOR_RESET}", file=sys.stderr)
 
 
 def fatal(msg: str) -> None:
-    print(f"\033[34mconfig: \033[31m{msg}\033[0m", file=sys.stderr)
+    print(f"{CSI}34mconfig: {CSI}39m{CSI}41m{msg}{COLOR_RESET}", file=sys.stderr)
 
 
 def init_config() -> None:
@@ -49,13 +50,14 @@ def init_config() -> None:
 
     if os.path.exists(user_dir):
         if not os.path.isdir(user_dir):
-            fatal("Please rename or remove the file {user_dir!r}.")
+            fatal(f"Please rename or remove the file {user_dir!r}.")
             sys.exit(CONFIG_ERROR)
     else:
         os.mkdir(user_dir)
 
     if os.path.isfile(config_file):
         if load_config():
+            # Stored at this point in order to put missing values in place.
             store_config()
             info("... Successfully updated user config.")
     else:
@@ -87,8 +89,10 @@ def load_config() -> bool:
     try:
         with open(config_file) as f:
             config = json.load(f)
-    except json.JSONDecodeError:
-        error("Error loading user config... Using defaults.")
+    except Exception as e:
+        error(
+            f"Failed to load user config ({type(e).__name__}: {e})... Using defaults."
+        )
         update_context_nav_keys(context_keys, nav, nav)
         return updated
 
@@ -96,8 +100,9 @@ def load_config() -> bool:
         c_version = config["version"]
         if gt(*[(*map(int, v.split(".")),) for v in (version, c_version)]):
             info("Updating user config...")
-            update_config(config, c_version)
-            updated = True
+            updated = update_config(config, c_version)
+            if not updated:
+                error("... Failed to update user config.")
     except KeyError:
         error("Config version not found... Please correct this manually.")
 
@@ -159,23 +164,30 @@ def store_config(*, default: bool = False) -> None:
         if keys:
             stored_keys[context] = keys
 
-    with open(config_file, "w") as f:
-        json.dump(
-            {
-                "version": version,
-                **{
-                    name: globals()["_" * default + f"{name.replace(' ', '_')}"]
-                    for name in config_options
+    try:
+        with open(config_file, "w") as f:
+            json.dump(
+                {
+                    "version": version,
+                    **{
+                        name: globals()["_" * default + f"{name.replace(' ', '_')}"]
+                        for name in config_options
+                    },
+                    "keys": stored_keys,
                 },
-                "keys": stored_keys,
-            },
-            f,
-            indent=4,
-        )
+                f,
+                indent=4,
+            )
+    except Exception as e:
+        error(f"Failed to write user config ({type(e).__name__}: {e}).")
 
 
-def update_config(config: Dict[str, Any], old_version: str):
-    """Updates the user config to latest version"""
+def update_config(config: Dict[str, Any], old_version: str) -> bool:
+    """Updates the user config to latest version
+
+    Returns:
+        ``True``, if successful. Otherwise, ``False``.
+    """
     # Must use the values directly, never reference the corresponding global variables,
     # as those might change later and will break updating since it's incremental
     #
@@ -194,6 +206,13 @@ def update_config(config: Dict[str, Any], old_version: str):
             ("['keys']['full-image']['Force Render'][1]", "F", "\u21e7F"),
             ("['keys']['full-grid-image']['Force Render'][1]", "F", "\u21e7F"),
         ],
+        "0.3": [
+            ("['font ratio']", 0.5, None),
+            ("['no multi']", NotImplemented, False),
+            ("['query timeout']", NotImplemented, 0.1),
+            ("['style']", NotImplemented, "auto"),
+            ("['swap win size']", NotImplemented, False),
+        ],
     }
 
     versions = tuple(changes)
@@ -216,11 +235,16 @@ def update_config(config: Dict[str, Any], old_version: str):
                         "the new default will be put in place."
                     )
 
-    os.replace(config_file, f"{config_file}.old")
-    info(f"Previous config file moved to '{config_file}.old'.")
     config["version"] = version
-    with open(config_file, "w") as f:
-        json.dump(config, f, indent=4)
+    try:
+        os.replace(config_file, f"{config_file}.old")
+    except OSError as e:
+        error(f"Failed to backup previous config file ({type(e).__name__}: {e})")
+        return False
+    else:
+        info(f"Previous config file has been moved to '{config_file}.old'.")
+
+    return True
 
 
 def update_context(name: str, keyset: Dict[str, list], update: Dict[str, list]) -> None:
@@ -318,7 +342,7 @@ def update_context_nav_keys(
 
 user_dir = os.path.join(os.path.expanduser("~"), ".term_image")
 config_file = os.path.join(user_dir, "config.json")
-version = "0.2"  # For config upgrades
+version = "0.3"  # For config upgrades
 
 _valid_keys = {*bytes(range(32, 127)).decode(), *urwid.escape._keyconv.values(), "esc"}
 _valid_keys.update(
@@ -365,15 +389,19 @@ for key in ("page up", "page down"):
 valid_keys.extend(("page up", "ctrl page up", "page down", "ctrl page down"))
 
 # Defaults
-_anim_cache = 100
-_cell_width = 30
-_checkers = None
-_font_ratio = 0.5
-_getters = 4
-_grid_renderers = 1
-_log_file = os.path.join(user_dir, "term_image.log")
-_max_notifications = 2
-_max_pixels = 2**22  # 2048x2048
+anim_cache = _anim_cache = 100
+cell_width = _cell_width = 30
+checkers = _checkers = None
+font_ratio = _font_ratio = None
+getters = _getters = 4
+grid_renderers = _grid_renderers = 1
+log_file = _log_file = os.path.join(user_dir, "term_image.log")
+max_notifications = _max_notifications = 2
+max_pixels = _max_pixels = 2**22  # 2048x2048
+no_multi = _no_multi = False
+query_timeout = _query_timeout = QUERY_TIMEOUT
+style = _style = "auto"
+swap_win_size = _swap_win_size = False
 
 _nav = {
     "Left": ["left", "\u25c0"],
@@ -468,15 +496,6 @@ _context_keys = {
 }
 # End of Defaults
 
-anim_cache = _anim_cache
-cell_width = _cell_width
-checkers = _checkers
-font_ratio = _font_ratio
-getters = _getters
-grid_renderers = _grid_renderers
-log_file = _log_file
-max_notifications = _max_notifications
-max_pixels = _max_pixels
 nav = deepcopy(_nav)
 context_keys = deepcopy(_context_keys)
 expand_key = context_keys["global"]["Key Bar"]
@@ -495,8 +514,8 @@ config_options = {
         "must be `null` or a non-negative integer",
     ),
     "font ratio": (
-        lambda x: isinstance(x, float) and x > 0.0,
-        "must be a float greater than zero",
+        lambda x: x is None or isinstance(x, float) and x > 0.0,
+        "must be `null` or a float greater than zero",
     ),
     "getters": (
         lambda x: isinstance(x, int) and x > 0,
@@ -526,4 +545,20 @@ config_options = {
         lambda x: isinstance(x, int) and x > 0,
         "must be an integer greater than zero",
     ),
+    "no multi": (
+        lambda x: isinstance(x, bool),
+        "must be a boolean",
+    ),
+    "query timeout": (
+        lambda x: isinstance(x, float) and x > 0.0,
+        "must be a float greater than zero",
+    ),
+    "style": (
+        lambda x: x in {"auto", "block", "iterm2", "kitty"},
+        "must be one of 'auto', 'block', 'iterm2', 'kitty'",
+    ),
+    "swap win size": (
+        lambda x: isinstance(x, bool),
+        "must be a boolean",
+    ),
 }
