@@ -5,17 +5,18 @@ from __future__ import annotations
 import logging as _logging
 import os
 from os.path import abspath, basename
-from shutil import get_terminal_size
 from time import sleep
 from types import FunctionType
-from typing import Any, Optional, Tuple
+from typing import Tuple
 
 import urwid
 
 from .. import __version__, logging
 from ..config import context_keys, expand_key
+from ..utils import get_terminal_size
 from . import main
 from .widgets import (
+    ImageCanvas,
     bottom_bar,
     confirmation,
     confirmation_overlay,
@@ -99,15 +100,15 @@ def display_context_help(context: str) -> None:
                     separator,
                     (
                         "weight",
-                        1,
+                        2,
                         urwid.Filler(
-                            urwid.Text(("default bold", f"{symbol}"), "center")
+                            urwid.Text(("default bold", f"{symbol} ({key})"), "center")
                         ),
                     ),
                     separator,
                     (
                         "weight",
-                        6,
+                        5,
                         urwid.Filler(
                             urwid.Text(("default bold", f"{description}"), "center")
                         ),
@@ -116,7 +117,7 @@ def display_context_help(context: str) -> None:
                 min_width=5,
             ),
         )
-        for action, (_, symbol, description, visible, _) in actions
+        for action, (key, symbol, description, visible, _) in actions
         if visible
     ]
 
@@ -125,9 +126,9 @@ def display_context_help(context: str) -> None:
         [
             ("weight", 3, line),
             (1, urwid.Filler(urwid.Text("\u253c"))),
-            ("weight", 1, line),
+            ("weight", 2, line),
             (1, urwid.Filler(urwid.Text("\u253c"))),
-            ("weight", 6, line),
+            ("weight", 5, line),
         ],
         min_width=5,
     )
@@ -142,9 +143,9 @@ def display_context_help(context: str) -> None:
                 [
                     ("weight", 3, line),
                     (1, urwid.Filler(urwid.Text("\u252c"))),
-                    ("weight", 1, line),
+                    ("weight", 2, line),
                     (1, urwid.Filler(urwid.Text("\u252c"))),
-                    ("weight", 6, line),
+                    ("weight", 5, line),
                 ],
                 min_width=5,
             ),
@@ -159,9 +160,9 @@ def display_context_help(context: str) -> None:
                     [
                         ("weight", 3, line),
                         (1, urwid.Filler(urwid.Text("\u2534"))),
-                        ("weight", 1, line),
+                        ("weight", 2, line),
                         (1, urwid.Filler(urwid.Text("\u2534"))),
-                        ("weight", 6, line),
+                        ("weight", 5, line),
                     ],
                     min_width=5,
                 ),
@@ -280,6 +281,8 @@ def set_confirmation(
     confirmation_overlay.bottom_w = bottom_widget
     main_widget.contents[0] = (confirmation_overlay, ("weight", 1))
 
+    main.ImageClass._clear_images()
+
 
 # Context Actions
 
@@ -296,52 +299,57 @@ def quit():
 
 @register_key(("global", "Key Bar"))
 def expand_collapse_keys():
-    global key_bar_is_collapsed
-
-    if expand_key_is_shown:
-        if key_bar_is_collapsed and key_bar_rows() > 1:
+    if expand._ti_shown:
+        if key_bar._ti_collapsed and key_bar_rows() > 1:
             expand.original_widget.set_text(f"\u25BC [{expand_key[1]}]")
             main_widget.contents[-1] = (
                 bottom_bar,
                 ("given", key_bar_rows()),
             )
-            key_bar_is_collapsed = False
-        elif not key_bar_is_collapsed:
+            key_bar._ti_collapsed = False
+            main.ImageClass._clear_images() and ImageCanvas.change()
+        elif not key_bar._ti_collapsed:
             expand.original_widget.set_text(f"\u25B2 [{expand_key[1]}]")
             main_widget.contents[-1] = (bottom_bar, ("given", 1))
-            key_bar_is_collapsed = True
+            key_bar._ti_collapsed = True
+            main.ImageClass._clear_images() and ImageCanvas.change()
 
 
 @register_key(("global", "Help"))
 def help():
     display_context_help(main.get_context())
+    main.ImageClass._clear_images()
 
 
 def resize():
-    global expand_key_is_shown
-
     cols = get_terminal_size()[0]
     rows = key_bar.original_widget.rows((cols,))
-    if expand_key_is_shown:
+    if expand._ti_shown:
         if rows == 1:
             bottom_bar.contents.pop()
-            expand_key_is_shown = False
+            expand._ti_shown = False
     elif rows > 1:
         bottom_bar.contents.append(
             (expand, ("given", len(expand.original_widget.text), False))
         )
-        expand_key_is_shown = True
+        expand._ti_shown = True
 
-    if not key_bar_is_collapsed:
+    if not key_bar._ti_collapsed:
+        new_rows = key_bar_rows()
+        if main_widget.contents[-1][1][1] != new_rows:
+            main.ImageClass._clear_images()
         main_widget.contents[-1] = (
             bottom_bar,
-            ("given", key_bar_rows()),
+            ("given", new_rows),
         )
 
 
 def key_bar_rows():
-    # Consider columns occupied by the expand key
-    cols = get_terminal_size()[0] - (5 + 2) * expand_key_is_shown
+    # Consider columns occupied by the expand key and the divider
+    cols = (
+        get_terminal_size()[0]
+        - (len(expand.original_widget.text) + 2) * expand._ti_shown
+    )
     return key_bar.original_widget.rows((cols,))
 
 
@@ -403,10 +411,13 @@ def open():
         main_widget.contents[0] = (view, ("weight", 1))
         set_image_view_actions()
 
+    main.ImageClass._clear_images()
+
 
 @register_key(("menu", "Back"))
 def back():
     main.displayer.send(main.BACK)
+    main.ImageClass._clear_images()
 
 
 # image
@@ -416,6 +427,8 @@ def maximize():
     main_widget.contents[0] = (view, ("weight", 1))
     set_image_view_actions()
 
+    main.ImageClass._clear_images()
+
 
 # image-grid
 @register_key(("image-grid", "Size-"))
@@ -427,6 +440,7 @@ def cell_width_dec():
         # Wait till GridRenderManager clears the cache
         while main.grid_change.is_set():
             pass
+        main.ImageClass._clear_images()
 
 
 @register_key(("image-grid", "Size+"))
@@ -438,25 +452,28 @@ def cell_width_inc():
         # Wait till GridRenderManager clears the cache
         while main.grid_change.is_set():
             pass
+        main.ImageClass._clear_images()
 
 
 @register_key(("image-grid", "Open"))
 def maximize_cell():
     main.set_context("full-grid-image")
     row = image_grid_box.base_widget.focus
-    image = (
+    image_w = (
         row.focus
         if isinstance(row, urwid.Columns)  # when maxcol >= cell_width
         else row
     ).original_widget.original_widget  # The Image is in a LineSquare in an AttrMap
 
-    image_box._w.contents[1][0].contents[1] = (image, ("weight", 1, True))
-    image_box.set_title(basename(image._image._source))
+    image_box._w.contents[1][0].contents[1] = (image_w, ("weight", 1, True))
+    image_box.set_title(basename(image_w._ti_image._source))
     main_widget.contents[0] = (image_box, ("weight", 1))
 
-    image_box.original_widget = image  # For image animation
-    if image._image._is_animated:
-        main.animate_image(image)
+    image_box.original_widget = image_w
+    if image_w._ti_image._is_animated:
+        main.animate_image(image_w)
+
+    main.ImageClass._clear_images()
 
 
 def set_image_grid_actions():
@@ -472,22 +489,18 @@ def set_image_grid_actions():
 def force_render_maximized_cell():
     # Will re-render immediately after processing input, since caching has been disabled
     # for `Image` widgets.
-    image = image_box._w.contents[1][0].contents[1][0]
-    if image._image._is_animated:
-        main.animate_image(image, True)
+    image_w = image_box._w.contents[1][0].contents[1][0]
+    if image_w._ti_image._is_animated:
+        main.animate_image(image_w, True)
     else:
-        image._force_render = True
+        image_w._ti_force_render = True
 
 
 # full-image, full-grid-image
 @register_key(("full-image", "Restore"), ("full-grid-image", "Back"))
 def restore():
-    # For image animation
-    if (
-        main.get_context() == "full-grid-image"
-        and image_box.original_widget._image._is_animated
-    ):
-        image_box.original_widget = placeholder
+    if main.get_context() == "full-grid-image":
+        image_box.original_widget = placeholder  # halt image and anim rendering
 
     main.set_prev_context()
     main_widget.contents[0] = (pile, ("weight", 1))
@@ -496,6 +509,8 @@ def restore():
     elif main.get_context() == "image":
         set_image_view_actions()
 
+    main.ImageClass._clear_images()
+
 
 # image, full-image
 @register_key(("image", "Prev"), ("full-image", "Prev"))
@@ -527,11 +542,11 @@ def next_image():
 def force_render():
     # Will re-render immediately after processing input, since caching has been disabled
     # for `Image` widgets.
-    image = main.menu_list[menu.focus_position - 1][1]
-    if image._image._is_animated:
-        main.animate_image(image, True)
+    image_w = main.menu_list[menu.focus_position - 1][1]
+    if image_w._ti_image._is_animated:
+        main.animate_image(image_w, True)
     else:
-        image._force_render = True
+        image_w._ti_force_render = True
 
 
 def set_image_view_actions(context: str = None):
@@ -661,13 +676,16 @@ def close():
 
 
 logger = _logging.getLogger(__name__)
-key_bar_is_collapsed = True
-expand_key_is_shown = True
 no_globals = {"global", "confirmation", "full-grid-image", "overlay"}
+key_bar._ti_collapsed = True
+expand._ti_shown = True
+
+# The annotations below are put in comments for compatibility with Python 3.7
+# as it doesn't allow names declared as `global` within functions to be annotated.
 
 # Use in the "confirmation" context. Set by `set_confirmation()`
-_confirm: Optional[Tuple[FunctionType, Tuple[Any]]] = None
-_cancel: Optional[Tuple[FunctionType, Tuple[Any]]] = None
+_confirm = None  #: Optional[Tuple[FunctionType, Tuple[Any]]]
+_cancel = None  #: Optional[Tuple[FunctionType, Tuple[Any]]]
 
 # Used for overlays
-_prev_view_widget: Optional[urwid.Widget] = None
+_prev_view_widget = None  #: Optional[urwid.Widget]
