@@ -4,7 +4,6 @@ from __future__ import annotations
 
 import logging as _logging
 import os
-from multiprocessing import Event as mp_Event
 from operator import mul
 from os.path import abspath, basename, islink
 from pathlib import Path
@@ -17,7 +16,6 @@ import urwid
 
 from .. import logging, notify, tui
 from ..config import context_keys, expand_key
-from ..image import ImageIterator, TermImage
 from .keys import (
     disable_actions,
     display_context_keys,
@@ -33,6 +31,7 @@ from .keys import (
 from .render import grid_render_queue
 from .widgets import (
     Image,
+    ImageCanvas,
     LineSquare,
     MenuEntry,
     image_box,
@@ -46,74 +45,36 @@ from .widgets import (
 )
 
 
-def animate_image(image: Image, forced_render: bool = False) -> None:
-    """Changes frames of an animated image"""
-    if NO_ANIMATION or (
-        mul(*image._image._original_size) > MAX_PIXELS and not forced_render
+def animate_image(image_w: Image, forced_render: bool = False) -> None:
+    """Initializes an animation."""
+    if not NO_ANIMATION and (
+        mul(*image_w._ti_image._original_size) <= MAX_PIXELS or forced_render
     ):
-        return
-
-    def next_frame(*_) -> None:
-        nonlocal last_alarm
-
-        loop.remove_alarm(last_alarm)
-        if (
-            image_box.original_widget is image
-            # In case you switch from and back to the image within one frame duration
-            and image._animator is animator
-            # The animator is not yet exhausted; repeat count is not yet zero
-            and image._animator.gi_frame
-            and (not forced_render or image._force_render)
-        ):
-            image._frame_changed = True
-            last_alarm = loop.set_alarm_in(frame_duration, next_frame)
-        # In case you switch from and back to the image within one frame duration
-        elif image._animator is animator:
-            # When you switch back and forth between an animated image and another
-            # image rapidly, all within one frame duration and the other image ends up
-            # as the current image, the last alarm from the first animation of the
-            # image in question and the first alarm from the second animation both
-            # meet the other image as the current one when they're triggered.
-            # So, they both try to clean up but the second alarm one meets nothing
-            # since the previous alarm had already cleaned up.
+        try:
+            del image_w._ti_anim_finished
+        except AttributeError:
             try:
-                image._animator.close()
-                del (
-                    image._animator,
-                    image._frame,
-                    image._frame_changed,
-                    image._frame_size_hash,
-                )
+                del image_w._ti_anim_ongoing
             except AttributeError:
                 pass
+        else:
+            if image_w._ti_canv:  # Hasn't been removed
+                image_w._ti_canv = None  # Deleting will break `ImageRenderManager`
 
-            # The above issue shouldn't apply to forced renders since no new
-            # animation is started until forced and that can't happen until the current
-            # forced animation is over.
-            # Going back to the image with the forced animation within one frame
-            # duration simply continues the ongoing animation.
-            #
-            # Also, see "Forced render" section of `.widgets.Image.render()`.
-            if forced_render:
-                del image._force_render
-                del image._forced_anim_size_hash
+        # Switched from this animated image earlier, to another animated image while
+        # AnimRenderManager was waiting on a frame's duration
+        # i.e the attributes didn't get to be removed by AnimRenderManager
+        try:
+            # If one fails, the rest shouldn't exist (removed by AnimRenderManager)
+            del image_w._ti_frame
+            del image_w._ti_force_render
+            del image_w._ti_forced_anim_size_hash
+        except AttributeError:
+            pass
 
-    frame_duration = FRAME_DURATION or image._image._frame_duration
-    animator = image._animator = ImageIterator(
-        image._image, REPEAT, f"1.1{image._alpha}", ANIM_CACHED
-    )._animator
-
-    # `Image.render()` checks for this. It has to be set here since `ImageIterator`
-    # doesn't set it until the first `next()` is called.
-    image._image._seek_position = 0
-
-    # Only needs to be set once for an animated image, not per frame
-    # See `Image.render()`
-    if forced_render:
-        image._force_render = True
-
-    image._frame_changed = True
-    last_alarm = loop.set_alarm_in(frame_duration, next_frame)
+        # Only needs to be set once for an animation, not per frame
+        if forced_render:
+            image_w._ti_force_render = True
 
 
 def display_images(
@@ -180,9 +141,10 @@ def display_images(
                 placeholder,
                 ("weight", 1, False),
             )
-            image_box.original_widget = placeholder  # For image animation
+            image_box.original_widget = placeholder  # halt image and anim rendering
             image_box.set_title("Image")
             view.original_widget = image_box
+            ImageClass._clear_images()
 
         elif pos == OPEN:  # Implements "menu::Open" action (for non-image entries)
             if prev_pos == -1:
@@ -281,8 +243,8 @@ def display_images(
                 image_box._w.contents[1][0].contents[1] = (value, ("weight", 1, False))
                 image_box.set_title(entry)
                 view.original_widget = image_box
-                image_box.original_widget = value  # For image animation
-                if value._image._is_animated:
+                image_box.original_widget = value
+                if value._ti_image._is_animated:
                     animate_image(value)
             else:  # Directory
                 grid_acknowledge.clear()
@@ -303,6 +265,7 @@ def display_images(
                     while grid_change.is_set():
                         pass
                     last_non_empty_grid_path = grid_path
+                image_box.original_widget = placeholder  # halt image and anim rendering
                 image_grid_box.set_title(grid_path + "/")
                 view.original_widget = image_grid_box
                 image_grid_box.base_widget._invalidate()
@@ -319,6 +282,8 @@ def display_images(
                 # GridScanner in the course of generating the display widget.
                 grid_acknowledge.wait()
 
+            ImageClass._clear_images() and ImageCanvas.change()
+
         prev_pos = pos
         pos = yield
         while pos == prev_pos:
@@ -446,7 +411,7 @@ def scan_dir(
         yield result, (
             entry.name,
             (
-                Image(TermImage.from_file(entry.path))
+                Image(ImageClass.from_file(entry.path))
                 if result == IMAGE
                 else ...
                 if result == DIR
@@ -522,7 +487,7 @@ def scan_dir_grid() -> None:
                 )
                 image_grid_box.base_widget._invalidate()
                 if page_not_complete:
-                    if len(grid_contents) <= image_grid_box.base_widget._page_ncell:
+                    if len(grid_contents) <= image_grid_box.base_widget._ti_page_ncell:
                         update_screen()
                     else:
                         page_not_complete = False
@@ -575,7 +540,7 @@ def scan_dir_menu() -> None:
                     )
                 )
                 if page_not_complete:
-                    if len(items) <= menu._height:
+                    if len(items) <= menu._ti_height:
                         update_screen()
                     else:
                         page_not_complete = False
@@ -589,7 +554,7 @@ def scan_dir_menu() -> None:
                     )
                 )
                 if page_not_complete:
-                    if len(items) <= menu._height:
+                    if len(items) <= menu._ti_height:
                         update_screen()
                     else:
                         page_not_complete = False
@@ -681,13 +646,16 @@ def update_menu(
 
 
 def update_screen():
-    """Triggers a screeen redraw.
+    """Triggers a screen redraw.
 
     Meant to be called from threads other than the thread in which the MainLoop is
     running.
     """
-    if not (interrupted.is_set() or quitting.is_set()):
+    try:
         os.write(update_pipe, b" ")
+    except OSError as e:
+        if e.errno != 9:
+            logging.log_exception("Screen update failed", logger)
 
 
 logger = _logging.getLogger(__name__)
@@ -699,9 +667,14 @@ grid_active = Event()
 grid_change = Event()
 grid_scan_done = Event()
 next_grid = Queue(1)
-_grid_list: Optional[list] = None
-grid_path: Optional[str] = None
-last_non_empty_grid_path: Optional[str] = None
+
+# The annotations below are put in comments for compatibility with Python 3.7
+# as it doesn't allow names declared as `global` within functions to be annotated.
+
+# # Set from within `display_images()`
+_grid_list = None  #: Optional[list]
+grid_path = None  #: Optional[str]
+last_non_empty_grid_path = None  #: Optional[str]
 
 # For menu scanning/listing
 menu_acknowledge = Event()
@@ -725,26 +698,23 @@ UNREADABLE = 1
 IMAGE = 2
 DIR = 3
 
-# Set by `update_menu()`
-menu_list: Optional[list] = None
-at_top_level: Optional[bool] = None
+# The annotations below are put in comments for compatibility with Python 3.7
+# as it doesn't allow names declared as `global` within functions to be annotated.
 
-# Intially set from `.__main__.main()`
-# Will be updated from `.logging.init_log()` if multiprocessing is enabled
-interrupted: Union[None, Event, mp_Event] = None
+# Set by `update_menu()`
+menu_list = None  #: Optional[list]
+at_top_level = None  #: Optional[bool]
 
 # Set from `.tui.init()`
+ImageClass: Optional[type] = None
 displayer: Optional[Generator[None, int, bool]] = None
 loop: Optional[tui.Loop] = None
 update_pipe: Optional[int] = None
 
 # # Corresponsing to command-line args
-ANIM_CACHED: Union[None, bool, int] = None
 DEBUG: Optional[bool] = None
-FRAME_DURATION: Optional[float] = None
 GRID_RENDERERS: Optional[int] = None
 MAX_PIXELS: Optional[int] = None
 NO_ANIMATION: Optional[bool] = None
-REPEAT: Optional[int] = None
 RECURSIVE: Optional[bool] = None
 SHOW_HIDDEN: Optional[bool] = None
